"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextInteractionImporterImplementation = void 0;
const moment = require("moment");
const log4js = require("log4js");
const fs = require("fs");
let Client = require('ftp');
const SSH2Promise = require("ssh2-promise");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const audio_service_1 = require("../services/audio.service");
const audio_transfer_implementation_1 = require("./audio-transfer.implementation");
const interaction_service_1 = require("../services/interaction.service");
const file_manager_1 = require("../shared/file-manager");
const aws_s3_implementation_1 = require("./aws-s3.implementation");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = '';
const CURRENT_IMPLEMENTATION = 'TextInteractionImporterImplementation';
class TextInteractionImporterImplementation {
    constructor(interaction) {
        this.interaction = null;
        this.interaction = interaction;
    }
    Get(connection) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.Get.name}] [vcc => ${this.interaction.vcc}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            switch (this.interaction.environmentType) {
                case 'ftp':
                    return this.GetFromFTP(connection)
                        .catch((err) => {
                        throw err?.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
                    });
                case 'sftp':
                    return this.GetFromSFTP()
                        .catch((err) => {
                        throw err?.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
                    });
                case 's3': return this.GetFromS3();
                default: throw 'SERVER_ERROR_ENVIRONMENT_TYPE_NOT_SUPPORTED';
            }
        })
            .then(result => {
            if (!Buffer.isBuffer(result.buffer)) {
                logger.error(`${logLabel}getting error, the result is not a buffer object`);
                throw ('SERVER_ERROR_CANNOT_GET_AUDIO_FILE_BUFFER');
            }
            return this.SaveRawData(result);
        })
            .then(_ => {
            return this.GetJsonFromRawData();
        })
            .catch(err => {
            logger.error(`${logLabel}Getting errorMessage:`, err);
            return Promise.reject(err);
        });
    }
    GetFromFTP(ftpClient) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetFromFTP.name}] [vcc::id => ${this.interaction.vcc}::${this.interaction.id}]: `;
        let filePaths = audio_service_1.GetInteractionFilePossiblePaths(this.interaction.id, this.interaction.relativePath, this.interaction.relativeFileName, this.interaction.ftpDirectory, 'TEXT');
        let environmentConfig = null;
        let ftp = null;
        if (!ftpClient || !ftpClient.connected) {
            environmentConfig = {
                host: this.interaction.apiHost,
                port: this.interaction.apiPort,
                user: this.interaction.apiUser,
                password: this.interaction.apiPassword,
                connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
            };
            ftp = new Client();
            ftp.connect(environmentConfig);
        }
        let genericResult = {};
        let startTime = moment();
        let timer;
        let minutesInterval = 1 * 60 * 1000;
        let maxWaitingMinutes = 5;
        timer = setInterval(() => {
            let currentTime = moment();
            let minutes = currentTime.diff(startTime, 'minutes');
            logger.info(`${logLabel}waiting ${minutes} minutes for ${this.interaction.id}`);
            if (minutes >= maxWaitingMinutes) {
                try {
                    logger.info(`${logLabel}closing FTP client after ${maxWaitingMinutes} minutos`);
                    ftpClient.end();
                }
                catch (e) {
                    logger.error(`${logLabel}failed closing FTP client`, e);
                }
            }
        }, minutesInterval);
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return new Promise((resolve, reject) => {
                ftp ?? (ftp = ftpClient);
                if (ftp) {
                    if (ftpClient)
                        this.SearchOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                    else {
                        ftp.on('greeting', (message) => {
                            logger.info('[AudioService::GetAudioFileFromFtp] Greeting event message:', message);
                        });
                        ftp.on('ready', () => {
                            this.SearchOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                        });
                        ftp.on('end', () => {
                            let eventMessage = 'FTP_CONNECTION_ENDED';
                            logger.warn(`${logLabel}getting eventMessage:`, eventMessage);
                            return resolve(eventMessage);
                        });
                        ftp.on('close', (hadError) => {
                            let eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                            logger.warn(`${logLabel}hadError:`, hadError);
                            return resolve(eventMessage);
                        });
                        ftp.on('error', (err) => {
                            logger.error(`${logLabel}NOT_FTP_CLIENT`, err);
                            if (filePaths.length)
                                this.SearchOnFtp(ftp, filePaths, !ftpClient, genericResult, resolve, reject);
                            else
                                return reject('NOT_FTP_CLIENT');
                        });
                    }
                }
                else {
                    logger.error(`${logLabel}SERVER_ERROR_FTP_NOT_CLIENT`);
                    return reject('SERVER_ERROR_NO_FTP_CLIENT');
                }
            });
        })
            .then(result => {
            logger.info(`${logLabel}Found file ${result} for interaction ${this.interaction.id}`);
            clearInterval(timer);
            return Promise.resolve({
                file: result,
                buffer: genericResult.extraContent
            });
        })
            .catch(err => {
            logger.error(`${logLabel}Getting errorMessage:`, err);
            clearInterval(timer);
            return Promise.reject(err);
        });
    }
    SearchOnFtp(clientFtp, filePaths, closeOnFile, genericResult, resolve, reject) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.SearchOnFtp.name}] [vcc => ${this.interaction.vcc}]: `;
        let ftpFilePath = filePaths.shift();
        logger.debug(`${logLabel}Searching:`, ftpFilePath, closeOnFile);
        clientFtp.get(ftpFilePath, (err, stream) => {
            logger.debug(`${logLabel}GET for:`, ftpFilePath);
            if (!err) {
                logger.debug(`${logLabel}No error condition`);
                genericResult.status = true;
                const bufs = [];
                stream.once('error', (err) => {
                    logger.error(`${logLabel}Once error:`, err);
                });
                stream.once('end', () => {
                    logger.info(`${logLabel}Once end`);
                });
                stream.once('close', () => {
                    logger.info(`${logLabel}Once close`);
                    if (closeOnFile)
                        clientFtp.end();
                    genericResult.extraContent = Buffer.concat(bufs);
                    resolve(ftpFilePath.split('/').pop());
                });
                stream.on('data', d => {
                    bufs.push(d);
                });
                stream.on('end', () => {
                    logger.info(`${logLabel}On end`);
                });
                stream.on('error', (err) => {
                    logger.error(`${logLabel}On error:`, err);
                });
                stream.on('close', () => {
                    logger.error(`${logLabel}On close`);
                });
            }
            else {
                logger.error(`${logLabel}Audiofile not found on:`, ftpFilePath);
                if (filePaths.length)
                    this.SearchOnFtp(clientFtp, filePaths, closeOnFile, genericResult, resolve, reject);
                else {
                    if (closeOnFile)
                        clientFtp.end();
                    genericResult.status = false;
                    genericResult.extraContent = 'SERVER_ERROR_AUDIO_FILE_GET_FTP';
                    reject(genericResult);
                }
            }
        });
    }
    GetFromSFTP(sftpClient) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetFromSFTP.name}] [vcc::id => ${this.interaction.vcc}::${this.interaction.id}]: `;
        let filePaths = audio_service_1.GetInteractionFilePossiblePaths(this.interaction.id, this.interaction.relativePath, this.interaction.relativeFileName, this.interaction.ftpDirectory, 'TEXT');
        let environmentConfig = null;
        let ssh2 = null;
        let sftp = null;
        if (!sftpClient) {
            environmentConfig = {
                host: this.interaction.apiHost,
                port: this.interaction.apiPort,
                username: this.interaction.apiUser,
                password: this.interaction.apiPassword,
                connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
            };
            if (environmentConfig.host === 's-b119f51bb73843339.server.transfer.eu-west-1.amazon') {
                environmentConfig.host = 's-b119f51bb73843339.server.transfer.eu-west-1.amazonaws.com';
                environmentConfig.privateKey = fs.readFileSync(`${config_1.PATH_FOLDER_SSL}/NewPrivatekey`, 'utf-8');
            }
            ssh2 = new SSH2Promise(environmentConfig);
        }
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (sftpClient || ssh2) {
                sftp = sftpClient || ssh2.sftp();
                return this.SearchOnSFTP(sftp, filePaths, !sftpClient, genericResult);
            }
            else {
                logger.error(`${logLabel}SERVER_ERROR_NOT_SFTP_CLIENT`);
                throw 'SERVER_ERROR_NOT_SFTP_CLIENT';
            }
        })
            .then(result => {
            if (genericResult.status === true) {
                logger.info(`${logLabel}Resolving general result`);
                return Promise.resolve({
                    audioFile: result,
                    buffer: genericResult.extraContent
                });
            }
            else {
                logger.error(`${logLabel}SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED`);
                throw 'SERVER_ERROR_GET_AUDIO_FROM_SFTP_FAILED';
            }
        })
            .catch(err => {
            logger.error(`${logLabel}General error: `, err);
            return Promise.reject(err);
        });
    }
    SearchOnSFTP(clientSftp, filePaths, closeOnFile, genericResult) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.SearchOnSFTP.name}] [vcc => ${this.interaction.vcc}]: `;
        let sftpFilePath = filePaths.shift();
        logger.debug(`${logLabel}Searching: ${sftpFilePath}`);
        return clientSftp.stat(sftpFilePath)
            .then(_ => {
            return clientSftp.createReadStream(sftpFilePath, { flags: 'r', mode: 0o666 })
                .then(stream => {
                if (stream) {
                    genericResult.status = true;
                    const bufs = [];
                    return new Promise((resolve, reject) => {
                        stream.on('error', err => {
                            logger.error(`${logLabel}SERVER_ERROR_CREATE_READ_STREAM_FAILED`);
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                        });
                        stream.once('close', () => {
                            try {
                                if (closeOnFile)
                                    clientSftp.close();
                                genericResult.extraContent = Buffer.concat(bufs);
                                return resolve(sftpFilePath.split('/').pop());
                            }
                            catch (e) {
                                return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                            }
                        });
                        stream.on('data', d => {
                            try {
                                bufs.push(d);
                            }
                            catch (e) {
                                return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                            }
                        });
                    });
                }
                else {
                    logger.error(`${logLabel}SERVER_ERROR_GET_AUDIO_STREAM_FAILED`);
                    throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
                }
            })
                .catch(err => {
                throw err;
            });
        })
            .catch(err => {
            logger.warn(`${logLabel}Audiofile not found on:`, sftpFilePath);
            logger.error(`${logLabel}General error: `, err);
            if (filePaths.length)
                return this.SearchOnSFTP(clientSftp, filePaths, closeOnFile, genericResult);
            else
                return Promise.resolve(null);
        });
    }
    GetFromS3(awsS3Impl) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetFromS3.name}] [vcc::id => ${this.interaction.vcc}::${this.interaction.id}]: `;
        let filePaths = audio_service_1.GetInteractionFilePossiblePaths(this.interaction.id, this.interaction.relativePath, this.interaction.relativeFileName, this.interaction.ftpDirectory, 'TEXT');
        let genericResult = {};
        let awsS3 = awsS3Impl ? awsS3Impl : new aws_s3_implementation_1.AwsS3Implementation(this.interaction.apiHost, this.interaction.apiUser, this.interaction.apiPassword, this.interaction.bucket);
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.SearchOnS3(awsS3, filePaths, genericResult);
        })
            .then(result => {
            logger.info(`${logLabel}Found file ${result} for interaction ${this.interaction.id}`);
            return Promise.resolve({
                audioFile: result,
                buffer: genericResult.extraContent
            });
        })
            .catch(err => {
            logger.error(`${logLabel}General error: `, err);
            return Promise.reject(err);
        });
    }
    SearchOnS3(awsS3impl, filePaths, genericResult) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.SearchOnS3.name}] [vcc => ${this.interaction.vcc}]: `;
        let awsS3FilePath = filePaths.shift();
        logger.debug(`${logLabel}Searching:`, awsS3FilePath);
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return awsS3impl.DownloadFile(awsS3FilePath);
        })
            .then(result => {
            if (result) {
                genericResult.status = true;
                const bufs = [];
                return new Promise((resolve, reject) => {
                    result.on('error', err => {
                        logger.error(`${logLabel}SERVER_ERROR_CREATE_READ_STREAM_FAILED`);
                        return reject('SERVER_ERROR_CREATE_READ_STREAM_FAILED');
                    });
                    result.once('close', () => {
                        try {
                            genericResult.extraContent = Buffer.concat(bufs);
                            return resolve(awsS3FilePath.split('/').pop());
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_CLOSE_FAILED');
                        }
                    });
                    result.on('data', d => {
                        try {
                            bufs.push(d);
                        }
                        catch (e) {
                            return reject('SERVER_ERROR_CREATE_READ_STREAM_ON_DATA_FAILED');
                        }
                    });
                });
            }
            else {
                logger.error(`${logLabel}SERVER_ERROR_GET_AUDIO_STREAM_FAILED`);
                throw 'SERVER_ERROR_GET_AUDIO_STREAM_FAILED';
            }
        })
            .catch(err => {
            logger.warn(`${logLabel}Audiofile not found on:`, awsS3FilePath);
            logger.error(`${logLabel}General error: `, err);
            if (filePaths.length)
                return this.SearchOnS3(awsS3impl, filePaths, genericResult);
            else
                return Promise.resolve(null);
        });
    }
    SaveRawData(rawData) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetJsonFromRawData.name}] [vcc => ${this.interaction.vcc}]: `;
        let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        let fileNamePath = audioTransfer.GetFileNamePathFromInteraction(this.interaction, 'json');
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return new Promise((resolve, reject) => {
                fs.open(fileNamePath, 'w', (err, fd) => {
                    if (err) {
                        logger.info(`${logLabel}getting error creating local audio file:`, err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, rawData.buffer, 0, rawData.buffer.length, null, (err) => {
                        if (err) {
                            logger.info(`${logLabel}getting error writing local audio file:`, err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, () => {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error(`${logLabel}Getting errorMessage:`, err);
            return Promise.reject(err);
        });
    }
    GetJsonFromRawData() {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetJsonFromRawData.name}] [vcc => ${this.interaction.vcc}]: `;
        let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        let fileNamePath = audioTransfer.GetFileNamePathFromInteraction(this.interaction, 'json');
        let messages = [];
        let format = '2.1';
        let results = [];
        let createdDate = moment.utc().format('YYYY-MM-DD HH:mm:ss');
        let metadata = {
            type: 'transcription',
            occ_data: {
                messages: [],
                interactionId: this.interaction.id
            },
            transcription_config: {
                diarization: 'channel',
                channel_diarization_labels: ['Agent', 'Caller'],
                language: this.interaction.language
            },
            created_at: createdDate
        };
        let job = {
            created_at: createdDate
        };
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(fileNamePath, 'utf-8', true);
        })
            .then(result => {
            let jsonResult = null;
            try {
                jsonResult = JSON.parse(result);
            }
            catch (err) {
                logger.error(`${logLabel}Error: `, err);
            }
            logger.info(`${logLabel}Getting messages from ${this.interaction.interactionType} interaction`);
            messages = interaction_service_1.GetTextMessagesFromTextData(this.interaction, jsonResult);
            return Promise.resolve(true);
        })
            .then(_ => {
            if (!messages.length)
                throw 'SERVER_ERROR_MESAGGES_NOT_FOUND';
            let firstMessageStartDate = null;
            messages.forEach((m, mi) => {
                let startDate = moment.unix(m.msgTime);
                if (!firstMessageStartDate)
                    firstMessageStartDate = startDate.set('milliseconds', 0);
                metadata.occ_data.messages.push(m);
                let tokenizedText = m.fullMsg.split(/(.+?)([\s,.!?;:)([\]]+)/);
                tokenizedText.forEach(tt => tt = tt.trim());
                tokenizedText = tokenizedText.filter(tt => tt.trim().length);
                tokenizedText.forEach(tt => {
                    let wordType = 'word';
                    if (tt.length === 1 && tt.match(/[`!@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?~]/))
                        wordType = 'punctuation';
                    results.push({
                        end_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                        channel: m.channel === 'Agent' ? 'Agent' : 'Caller',
                        occ_data: {
                            msgTime: m.msgTime,
                            msgDate: startDate.utc().format('YYYY-MM-DD HH:mm:ss.000'),
                            word_number: 0,
                            msgNumber: mi + 1
                        },
                        start_time: startDate.diff(firstMessageStartDate, 'milliseconds') / 1000,
                        type: wordType,
                        alternatives: [{
                                confidence: 1,
                                content: tt,
                                language: this.interaction.language
                            }]
                    });
                });
                results.forEach((r, i) => r.occ_data.word_number = (i + 1));
            });
            return Promise.resolve(true);
        })
            .then(_ => {
            let objectResult = {
                format,
                metadata,
                results,
                job
            };
            return file_manager_1.WriteFileContent(fileNamePath, JSON.stringify(objectResult), 'utf-8', false);
        })
            .then(_ => {
            this.interaction.fileName = fileNamePath.split('/').pop();
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error(`${logLabel}Getting errorMessage:`, err);
            return Promise.reject(err);
        });
    }
}
exports.TextInteractionImporterImplementation = TextInteractionImporterImplementation;
